(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
	typeof define === 'function' && define.amd ? define(['exports'], factory) :
	(factory((global.typedash = global.typedash || {})));
}(this, (function (exports) { 'use strict';

function curry(f, n) {
    var argsArray = argumentsArray(arguments);
    if (typeof n === 'undefined') {
        argsArray[1] = f.length;
    }
    if (n === argsArray.length - 2) {
        return f.apply(undefined, argsArray.slice(2));
    }
    return function () {
        return curry.apply(undefined, concat(argsArray, argumentsArray(arguments)));
    };
}
/***
 * Function that do nothing
 */
/* tslint:disable-next-line no-empty */
function noop() {
}
/***
 * Function for identity enumeration. Returns what passed to it
 * @param i
 * @returns {T}
 */
function identity(i) {
    return i;
}
/***
 *Creates a debounced function that delays invoking func until after wait milliseconds have elapsed
 * since the last time the debounced function was invoked.
 * The func is invoked with the last arguments provided to the debounced function.
 * @param func
 * @param wait
 * @param immediate indicate whether func should be invoked on the leading and/or trailing edge of the wait timeout.
 * @returns {Function}
 */
function debounce(func, wait, immediate) {
    if (immediate === void 0) { immediate = false; }
    var timeout;
    return function () {
        var context = this;
        var args = arguments;
        var later = function () {
            timeout = null;
            if (!immediate)
                func.apply(context, args);
        };
        var callNow = immediate && !timeout;
        clearTimeout(timeout);
        timeout = setTimeout(later, wait);
        if (callNow)
            func.apply(context, args);
    };
}
/***
 * Creates a function that is restricted to invoking func once.
 * Repeat calls to the function return the value of the first invocation.
 * The func is invoked with the this binding and arguments of the created function.
 * @param func
 * @returns {T}
 */
function once(func) {
    var called = false;
    var res;
    return function () {
        if (!called) {
            called = true;
            res = func.apply(this, arguments);
        }
        return res;
    };
}
/***
 * Creates a function that memoizes the result of func.
 * The func is invoked with the this binding of the memoized function.
 * @param func
 * @returns {T}
 */
function memoize(func) {
    var stringifyJson = JSON.stringify;
    var cache = {};
    return function () {
        var hash = stringifyJson(arguments);
        return (hash in cache) ? cache[hash] : cache[hash] = func.apply(this, arguments);
    };
}
/***
 * Creates a delayed function that delays invoking func until called by schedulerFn.
 * The func is invoked with the last arguments provided to the delayed function.
 * @param schedulerFn
 * @param fn
 * @returns {T}
 */
function delayed(schedulerFn, fn) {
    var ticking = false;
    var ctx = this;
    var lastArgs = [];
    function handle() {
        fn.apply(ctx, lastArgs);
        ticking = false;
    }
    return function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        if (!ticking) {
            schedulerFn(handle);
        }
        ticking = true;
        lastArgs = args;
        ctx = this;
    };
}

var arrayPrototype = Array.prototype;
/***
 * Filter items by predicate
 * @param array
 * @param callbackfn
 * @returns {T[]}
 */
function filter(array, callbackfn) {
    return array.filter(callbackfn);
}
/***
 * Iterate over array with `callbackfn`
 * @param array
 * @param callbackfn
 * @returns {T[]}
 */
function forEach(array, callbackfn) {
    array.forEach(callbackfn);
    return array;
}
/***
 * Get index of item in array
 * @param array
 * @param item
 * @returns {number} -1 if item is not found
 */
function indexOf(array, item) {
    return array.indexOf(item);
}
/***
 * Creates an array with all falsey values removed.
 * The values `false`, `null`, `0`, `""`, `undefined`, and `NaN` are falsey.
 * @param array array to compact.
 * @returns {T[]} Returns the new array of filtered values.
 */
function compact(array) {
    return filter(array, identity);
}
/***
 * Return array from arguments
 * @param argsObject
 * @returns {any[]}
 */
function argumentsArray(argsObject) {
    return arrayPrototype.slice.call(argsObject, 0);
}
/***
 * Creates a new array concatenating array with any additional arrays and/or values.
 * @param item item or array to concatenate
 * @returns {T[]} Returns the new concatenated array.
 */
function concat() {
    var item = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        item[_i] = arguments[_i];
    }
    return arrayPrototype.concat.apply([], item);
}
/***
 * Flattens array a single level deep.
 * @param array The array to flatten.
 * @returns {T[]} Return Flattened array.
 */
function flatten(array) {
    return concat.apply(void 0, [[]].concat(array));
}
/***
 * Creates a duplicate-free version of an array,
 * using `indexOf` for equality comparisons,
 * in which only the first occurrence of each element is kept.
 * The order of result values is determined by the order they occur in the array.
 * @param array The array to inspect.
 * @returns {T[]} Duplicate free array
 */
function uniq(array) {
    return filter(array, function (item, index, array) { return indexOf(array, item) === index; });
}
/***
 * Creates an array excluding all given values using `indexOf` for equality comparisons.
 * @param array The array to inspect.
 * @param remove The array to exclude.
 * @returns {T[]} Returns the new array of filtered values.
 */
function without(array, remove) {
    return filter(array, function (item) { return !contains(remove, item); });
}
/***
 * Check if item is in array using `indexOf`
 * @param array The array to inspect
 * @param item The item to check
 * @returns {boolean} Return `true` if item is in array
 */
function contains(array, item) {
    return indexOf(array, item) !== -1;
}
/***
 * Creates an symmetric difference array of values not included in the other given arrays.
 * The order and references of result values are determined by the first array.
 * @param array1 The first array
 * @param array2 The second array
 * @returns {[T]} Returns values not included in the other given arrays.
 */
function difference(array1, array2) {
    return concat(without(array1, array2), without(array2, array1));
}
/***
 * Check if argument is an array
 * @param item
 * @returns {boolean}
 */
function isArray(item) {
    return Array.isArray(item);
}

/***
 * Get object keys from `item`
 * @param item The object to get keys from
 * @returns {string[]} Object keys
 */
function keys(item) {
    return Object.keys(item);
}
/***
 * Is object empty (no keys)
 * @param item The object to check
 * @returns {boolean}
 */
function isEmpty(item) {
    return !item || (isArray(item) && !item.length) || !keys(item).length;
}
/***
 * Get object key values from `item`
 * @param item The object to get keys from
 * @returns {V[]} Object values
 */
function values(item) {
    return keys(item).reduce(function (res, key) { return concat(res, item[key]); }, []);
}
/***
 * Create new object with mapped key/value from `item`
 * @param item
 * @param mapKeyFn Function to map keys
 * @param mapValueFn Function to map values
 * @returns {[p:string]: V}
 */
function mapKeyValues(item, mapKeyFn, mapValueFn) {
    return keys(item).reduce(function (res, key) {
        res[mapKeyFn(key)] = mapValueFn(item[key]);
        return res;
    }, {});
}
/***
 * Create new object with mapped keys from `item`
 * @param item
 * @param mapKeyFn Function to map key
 * @returns {{[p: string]: V}}
 */
function mapKeys(item, mapKeyFn) {
    return mapKeyValues(item, mapKeyFn, identity);
}
/***
 * Create new object with mapped values from `item`
 * @param item
 * @param mapValueFn Function to map values
 * @returns {{[p: string]: R}}
 */
function mapValues(item, mapValueFn) {
    return mapKeyValues(item, identity, mapValueFn);
}
/***
 * Reduce `item` keys and creates new object
 * @param item
 * @param reduceFn Reduce function
 * @returns {{[p: string]: any}}
 */
function reduce(item, reduceFn) {
    return keys(item).reduce(function (res, key) {
        res[key] = reduceFn(key, item[key]);
        return res;
    }, {});
}
/***
 * Filter `item` keys and create new object
 * @param item
 * @param filterFn
 * @returns {Partial<any>}
 */
function filter$1(item, filterFn) {
    return keys(item).reduce(function (res, key) {
        if (filterFn(key)) {
            res[key] = item[key];
        }
        return res;
    }, {});
}
/***
 * Omit keys from `item` and return new object
 * @param item
 * @param keys The keys to exclude
 * @returns {Partial<T>}
 */
function omit(item, keys) {
    return filter$1(item, function (key) { return !contains(keys, key); });
}
/***
 * Pick keys from `item` and return new object
 * @param item
 * @param keys The keys to include
 * @returns {Partial<T>}
 */
function pick(item, keys) {
    return filter$1(item, function (key) { return contains(keys, key); });
}
var toString = Object.prototype.toString;
function isMergeableObject(val) {
    return val && typeof val === 'object'
        && toString.call(val) !== '[object RegExp]'
        && toString.call(val) !== '[object Date]';
}
function emptyTarget(val) {
    return isArray(val) ? [] : {};
}
function cloneIfNecessary(value) {
    return (isMergeableObject(value)) ? merge(emptyTarget(value), value) : value;
}
function arrayMerge(target, source) {
    var destination = target.slice();
    forEach(source, function (e, i) {
        if (typeof destination[i] === 'undefined') {
            destination[i] = cloneIfNecessary(e);
        }
        else if (isMergeableObject(e)) {
            destination[i] = merge(target[i], e);
        }
        else if (target.indexOf(e) === -1) {
            destination.push(cloneIfNecessary(e));
        }
    });
    return destination;
}
function mergeObject(target, source) {
    var destination = {};
    if (isMergeableObject(target)) {
        forEach(keys(target), function (key) {
            destination[key] = cloneIfNecessary(target[key]);
        });
    }
    forEach(keys(source), function (key) {
        if (!isMergeableObject(source[key]) || !target[key]) {
            destination[key] = cloneIfNecessary(source[key]);
        }
        else {
            destination[key] = merge(target[key], source[key]);
        }
    });
    return destination;
}
/***
 * This method is like Object.assign except that it recursively merges own and inherited enumerable string
 * keyed properties of source objects into the destination object.
 * Source properties that resolve to undefined are skipped if a destination value exists.
 * Array and plain object properties are merged recursively.
 * Other objects and value types are overridden by assignment.
 * Source objects are applied from left to right.
 * Subsequent sources overwrite property assignments of previous sources.
 * @param target
 * @param source
 * @returns {object|Array}
 */
function merge(target, source) {
    if (isArray(source)) {
        return isArray(target) ? arrayMerge(target, source) : cloneIfNecessary(source);
    }
    else {
        return mergeObject(target, source);
    }
}
/***
 * Creates a patch for `old` using values from `patch`
 * @example `patch({a:1,b:2}, {a:1,b:3,c:4}) === {b:3,c:4}`
 * @param old
 * @param patch
 * @returns {Partial<Partial<any>>}
 */
function patch(old, patch) {
    return filter$1(patch, function (key) { return patch[key] !== old[key]; });
}
/***
 * Check object keys equality
 * @param objA
 * @param objB
 * @returns {boolean}
 */
function shallowEqual(objA, objB) {
    if (objA === objB) {
        return true;
    }
    if (typeof objA !== 'object' || !objA ||
        typeof objB !== 'object' || !objB) {
        return false;
    }
    var keysA = keys(objA);
    var keysB = keys(objB);
    if (keysA.length !== keysB.length) {
        return false;
    }
    var bHasOwnProperty = Object.prototype.hasOwnProperty.bind(objB);
    // Test for A's keys different from B.
    for (var idx = 0; idx < keysA.length; idx++) {
        var key = keysA[idx];
        if (!bHasOwnProperty(key)) {
            return false;
        }
        var valueA = objA[key];
        var valueB = objB[key];
        if (valueA !== valueB) {
            return false;
        }
    }
    return true;
}

var asciiWords = /[^\x00-\x2f\x3a-\x40\x5b-\x60\x7b-\x7f]+/g;
/***
 * Spit string into words, unicode is not supported
 * @param str The string to split
 * @returns {RegExpMatchArray|[]} Matched words
 */
function words(str) {
    return (str && str.match(asciiWords)) || [];
}
function mapWords(str, wordMapFn, join) {
    if (join === void 0) { join = ''; }
    return words(str).map(wordMapFn).join(join);
}
/***
 * Convert string to lowerCase
 * @param s The string to convert
 * @returns {string} Returns string in lowercase
 */
function lowerCase(s) {
    return s.toLowerCase();
}
function caseStr(casing, str) {
    return mapWords(str, lowerCase, casing);
}
/***
 * Convert string to kebab-case
 * @param s The string to convert
 * @returns {string} Returns string in kebab-case
 */
var kebabCase = curry(caseStr)('-');
/***
 * Convert string to snake_case
 * @param s The string to convert
 * @returns {string} Returns string in snake_case
 */
var snakeCase = curry(caseStr)('_');
/***
 * Capitalize first char of string
 * @param str
 * @returns {string} Returns string with first char capitalized
 */
function capitalize(str) {
    return str[0].toUpperCase() + str.slice(1);
}
/***
 * Convert string to camelCase
 * @param str The string to convert
 * @returns {string} Returns string in camelCase
 */
function camelCase(str) {
    return mapWords(str, function (word, i) {
        word = lowerCase(word);
        return i === 0 ? word : capitalize(word);
    });
}
/***
 * Removes leading and trailing whitespace or specified characters from string.
 * @param str The string to trim.
 * @returns {string} Trimmed string
 */
function trim(str) {
    return str.trim();
}
/***
 * Convert wildcard string to RegExp. Like `hel*o` becames `/hel.*o/`
 * @param wildcardPattern pattern to convert
 * @param flags flags for created regex 'igm'
 * @returns {RegExp} Constructed regexp
 */
function wildcardToRegExp(wildcardPattern, flags) {
    var regexMatch = /^\/(.+)\/([im]+)?$/.exec(wildcardPattern);
    if (regexMatch) {
        return new RegExp(regexMatch[1], regexMatch[2] || flags);
    }
    var pattern = wildcardPattern
        .replace(/[\-\[\]\/{}()?.\\^$|]/g, '\\$&')
        .replace(/\*/g, '.*?')
        .replace(/\+/g, '.+?');
    return new RegExp("^" + pattern + "$", flags);
}
/***
 * Compile string template into template function.
 * Template language is similar to template strings in es6 `Hello ${name}`.
 * Doesn't support anything but property name inside `${}`
 * @example Use `template('Hello ${name}')(templateObject({name:'Bob'}))==='Hello Bob'`
 * @param template The template string
 * @returns {(data:<T>(key: keyof T) => string)=>string} Compiled template function.
 */
function template(template) {
    var sanitized = template.split(/(\${([\s]*[^;\s{]+[\s]*)})/g);
    var fn = 'return [';
    for (var i = 0; i < sanitized.length; i++) {
        if (sanitized[i][0] === '$') {
            fn += "$d('" + sanitized[++i] + "'),";
        }
        else {
            fn += "\"" + sanitized[i].replace('"', '\\"') + "\",";
        }
    }
    return Function('$d', fn + "]");
}
/***
 * Function to use with `template` return object keys by template key
 * @param value
 * @param nullValue
 * @returns {(key:keyof T)=>string?}
 */
function templateObject(value, nullValue) {
    return function (key) {
        return value[key] || nullValue;
    };
}

exports.forEach = forEach;
exports.indexOf = indexOf;
exports.compact = compact;
exports.argumentsArray = argumentsArray;
exports.concat = concat;
exports.flatten = flatten;
exports.uniq = uniq;
exports.without = without;
exports.contains = contains;
exports.difference = difference;
exports.isArray = isArray;
exports.keys = keys;
exports.isEmpty = isEmpty;
exports.values = values;
exports.mapKeyValues = mapKeyValues;
exports.mapKeys = mapKeys;
exports.mapValues = mapValues;
exports.reduce = reduce;
exports.filter = filter$1;
exports.omit = omit;
exports.pick = pick;
exports.merge = merge;
exports.patch = patch;
exports.shallowEqual = shallowEqual;
exports.curry = curry;
exports.noop = noop;
exports.identity = identity;
exports.debounce = debounce;
exports.once = once;
exports.memoize = memoize;
exports.delayed = delayed;
exports.words = words;
exports.lowerCase = lowerCase;
exports.kebabCase = kebabCase;
exports.snakeCase = snakeCase;
exports.capitalize = capitalize;
exports.camelCase = camelCase;
exports.trim = trim;
exports.wildcardToRegExp = wildcardToRegExp;
exports.template = template;
exports.templateObject = templateObject;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=typedash.dev.js.map
