import { identity } from './functions';
var arrayPrototype = Array.prototype;
/***
 * Filter items by predicate
 * @param array
 * @param callbackfn
 * @returns {T[]}
 */
function filter(array, callbackfn) {
    return array.filter(callbackfn);
}
/***
 * Iterate over array with `callbackfn`
 * @param array
 * @param callbackfn
 * @returns {T[]}
 */
export function forEach(array, callbackfn) {
    array.forEach(callbackfn);
    return array;
}
/***
 * Get index of item in array
 * @param array
 * @param item
 * @returns {number} -1 if item is not found
 */
export function indexOf(array, item) {
    return array.indexOf(item);
}
/***
 * Creates an array with all falsey values removed.
 * The values `false`, `null`, `0`, `""`, `undefined`, and `NaN` are falsey.
 * @param array array to compact.
 * @returns {T[]} Returns the new array of filtered values.
 */
export function compact(array) {
    return filter(array, identity);
}
/***
 * Return array from arguments
 * @param argsObject
 * @returns {any[]}
 */
export function argumentsArray(argsObject) {
    return arrayPrototype.slice.call(argsObject, 0);
}
/***
 * Creates a new array concatenating array with any additional arrays and/or values.
 * @param item item or array to concatenate
 * @returns {T[]} Returns the new concatenated array.
 */
export function concat() {
    var item = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        item[_i] = arguments[_i];
    }
    return arrayPrototype.concat.apply([], item);
}
/***
 * Flattens array a single level deep.
 * @param array The array to flatten.
 * @returns {T[]} Return Flattened array.
 */
export function flatten(array) {
    return concat.apply(void 0, [[]].concat(array));
}
/***
 * Creates a duplicate-free version of an array,
 * using `indexOf` for equality comparisons,
 * in which only the first occurrence of each element is kept.
 * The order of result values is determined by the order they occur in the array.
 * @param array The array to inspect.
 * @returns {T[]} Duplicate free array
 */
export function uniq(array) {
    return filter(array, function (item, index, array) { return indexOf(array, item) === index; });
}
/***
 * Creates an array excluding all given values using `indexOf` for equality comparisons.
 * @param array The array to inspect.
 * @param remove The array to exclude.
 * @returns {T[]} Returns the new array of filtered values.
 */
export function without(array, remove) {
    return filter(array, function (item) { return !contains(remove, item); });
}
/***
 * Check if item is in array using `indexOf`
 * @param array The array to inspect
 * @param item The item to check
 * @returns {boolean} Return `true` if item is in array
 */
export function contains(array, item) {
    return indexOf(array, item) !== -1;
}
/***
 * Creates an symmetric difference array of values not included in the other given arrays.
 * The order and references of result values are determined by the first array.
 * @param array1 The first array
 * @param array2 The second array
 * @returns {[T]} Returns values not included in the other given arrays.
 */
export function difference(array1, array2) {
    return concat(without(array1, array2), without(array2, array1));
}
/***
 * Check if argument is an array
 * @param item
 * @returns {boolean}
 */
export function isArray(item) {
    return Array.isArray(item);
}
//# sourceMappingURL=arrays.js.map