import { argumentsArray, concat } from './arrays';
export function curry(f, n) {
    var argsArray = argumentsArray(arguments);
    if (typeof n === 'undefined') {
        argsArray[1] = f.length;
    }
    if (n === argsArray.length - 2) {
        return f.apply(undefined, argsArray.slice(2));
    }
    return function () {
        return curry.apply(undefined, concat(argsArray, argumentsArray(arguments)));
    };
}
/***
 * Function that do nothing
 */
/* tslint:disable-next-line no-empty */
export function noop() {
}
/***
 * Function for identity enumeration. Returns what passed to it
 * @param i
 * @returns {T}
 */
export function identity(i) {
    return i;
}
/***
 *Creates a debounced function that delays invoking func until after wait milliseconds have elapsed
 * since the last time the debounced function was invoked.
 * The func is invoked with the last arguments provided to the debounced function.
 * @param func
 * @param wait
 * @param immediate indicate whether func should be invoked on the leading and/or trailing edge of the wait timeout.
 * @returns {Function}
 */
export function debounce(func, wait, immediate) {
    if (immediate === void 0) { immediate = false; }
    var timeout;
    return function () {
        var context = this;
        var args = arguments;
        var later = function () {
            timeout = null;
            if (!immediate)
                func.apply(context, args);
        };
        var callNow = immediate && !timeout;
        clearTimeout(timeout);
        timeout = setTimeout(later, wait);
        if (callNow)
            func.apply(context, args);
    };
}
/***
 * Creates a function that is restricted to invoking func once.
 * Repeat calls to the function return the value of the first invocation.
 * The func is invoked with the this binding and arguments of the created function.
 * @param func
 * @returns {T}
 */
export function once(func) {
    var called = false;
    var res;
    return function () {
        if (!called) {
            called = true;
            res = func.apply(this, arguments);
        }
        return res;
    };
}
/***
 * Creates a function that memoizes the result of func.
 * The func is invoked with the this binding of the memoized function.
 * @param func
 * @returns {T}
 */
export function memoize(func) {
    var stringifyJson = JSON.stringify;
    var cache = {};
    return function () {
        var hash = stringifyJson(arguments);
        return (hash in cache) ? cache[hash] : cache[hash] = func.apply(this, arguments);
    };
}
/***
 * Creates a delayed function that delays invoking func until called by schedulerFn.
 * The func is invoked with the last arguments provided to the delayed function.
 * @param schedulerFn
 * @param fn
 * @returns {T}
 */
export function delayed(schedulerFn, fn) {
    var ticking = false;
    var ctx = this;
    var lastArgs = [];
    function handle() {
        fn.apply(ctx, lastArgs);
        ticking = false;
    }
    return function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        if (!ticking) {
            schedulerFn(handle);
        }
        ticking = true;
        lastArgs = args;
        ctx = this;
    };
}
//# sourceMappingURL=functions.js.map