/***
 * Get object keys from `item`
 * @param item The object to get keys from
 * @returns {string[]} Object keys
 */
export declare function keys<T extends object>(item: T): (keyof T)[];
/***
 * Is object empty (no keys)
 * @param item The object to check
 * @returns {boolean}
 */
export declare function isEmpty(item?: Object | Array<any>): boolean;
/***
 * Get object key values from `item`
 * @param item The object to get keys from
 * @returns {V[]} Object values
 */
export declare function values<V, T extends {
    [key: string]: V;
}>(item: T): V[];
/***
 * Create new object with mapped key/value from `item`
 * @param item
 * @param mapKeyFn Function to map keys
 * @param mapValueFn Function to map values
 * @returns {[p:string]: V}
 */
export declare function mapKeyValues<V, T extends {
    [key: string]: V;
}, R>(item: T, mapKeyFn: (key: (keyof T)) => string, mapValueFn: (value: V) => R): {
    [key: string]: R;
};
/***
 * Create new object with mapped keys from `item`
 * @param item
 * @param mapKeyFn Function to map key
 * @returns {{[p: string]: V}}
 */
export declare function mapKeys<V, T extends {
    [key: string]: V;
}>(item: T, mapKeyFn: (key: (keyof T)) => string): {
    [key: string]: V;
};
/***
 * Create new object with mapped values from `item`
 * @param item
 * @param mapValueFn Function to map values
 * @returns {{[p: string]: R}}
 */
export declare function mapValues<V, T extends {
    [key: string]: V;
}, R>(item: T, mapValueFn: <R>(value: V) => R): {
    [key: string]: R;
};
/***
 * Reduce `item` keys and creates new object
 * @param item
 * @param reduceFn Reduce function
 * @returns {{[p: string]: any}}
 */
export declare function reduce<V, T extends {
    [key: string]: V;
}, R>(item: T, reduceFn: (key: (keyof T), value: V) => R): {
    [key: string]: R;
};
/***
 * Filter `item` keys and create new object
 * @param item
 * @param filterFn
 * @returns {Partial<any>}
 */
export declare function filter<T extends object>(item: T, filterFn: (key: (keyof T)) => boolean): Partial<T>;
/***
 * Omit keys from `item` and return new object
 * @param item
 * @param keys The keys to exclude
 * @returns {Partial<T>}
 */
export declare function omit<T extends object>(item: T, keys: (keyof T | string)[]): Partial<T>;
/***
 * Pick keys from `item` and return new object
 * @param item
 * @param keys The keys to include
 * @returns {Partial<T>}
 */
export declare function pick<T extends object>(item: T, keys: (keyof T | string)[]): Partial<T>;
/***
 * This method is like Object.assign except that it recursively merges own and inherited enumerable string
 * keyed properties of source objects into the destination object.
 * Source properties that resolve to undefined are skipped if a destination value exists.
 * Array and plain object properties are merged recursively.
 * Other objects and value types are overridden by assignment.
 * Source objects are applied from left to right.
 * Subsequent sources overwrite property assignments of previous sources.
 * @param target
 * @param source
 * @returns {object|Array}
 */
export declare function merge<T, S>(target: T, source: S): T & S | Array<T | S>;
/***
 * Creates a patch for `old` using values from `patch`
 * @example `patch({a:1,b:2}, {a:1,b:3,c:4}) === {b:3,c:4}`
 * @param old
 * @param patch
 * @returns {Partial<Partial<any>>}
 */
export declare function patch<T extends object>(old: T, patch: Partial<T>): Partial<T>;
/***
 * Check object keys equality
 * @param objA
 * @param objB
 * @returns {boolean}
 */
export declare function shallowEqual(objA: Object, objB: Object): boolean;
