import { concat, contains, forEach, isArray } from './arrays';
import { identity } from './functions';
/***
 * Get object keys from `item`
 * @param item The object to get keys from
 * @returns {string[]} Object keys
 */
export function keys(item) {
    return Object.keys(item);
}
/***
 * Is object empty (no keys)
 * @param item The object to check
 * @returns {boolean}
 */
export function isEmpty(item) {
    return !item || (isArray(item) && !item.length) || !keys(item).length;
}
/***
 * Get object key values from `item`
 * @param item The object to get keys from
 * @returns {V[]} Object values
 */
export function values(item) {
    return keys(item).reduce(function (res, key) { return concat(res, item[key]); }, []);
}
/***
 * Create new object with mapped key/value from `item`
 * @param item
 * @param mapKeyFn Function to map keys
 * @param mapValueFn Function to map values
 * @returns {[p:string]: V}
 */
export function mapKeyValues(item, mapKeyFn, mapValueFn) {
    return keys(item).reduce(function (res, key) {
        res[mapKeyFn(key)] = mapValueFn(item[key]);
        return res;
    }, {});
}
/***
 * Create new object with mapped keys from `item`
 * @param item
 * @param mapKeyFn Function to map key
 * @returns {{[p: string]: V}}
 */
export function mapKeys(item, mapKeyFn) {
    return mapKeyValues(item, mapKeyFn, identity);
}
/***
 * Create new object with mapped values from `item`
 * @param item
 * @param mapValueFn Function to map values
 * @returns {{[p: string]: R}}
 */
export function mapValues(item, mapValueFn) {
    return mapKeyValues(item, identity, mapValueFn);
}
/***
 * Reduce `item` keys and creates new object
 * @param item
 * @param reduceFn Reduce function
 * @returns {{[p: string]: any}}
 */
export function reduce(item, reduceFn) {
    return keys(item).reduce(function (res, key) {
        res[key] = reduceFn(key, item[key]);
        return res;
    }, {});
}
/***
 * Filter `item` keys and create new object
 * @param item
 * @param filterFn
 * @returns {Partial<any>}
 */
export function filter(item, filterFn) {
    return keys(item).reduce(function (res, key) {
        if (filterFn(key)) {
            res[key] = item[key];
        }
        return res;
    }, {});
}
/***
 * Omit keys from `item` and return new object
 * @param item
 * @param keys The keys to exclude
 * @returns {Partial<T>}
 */
export function omit(item, keys) {
    return filter(item, function (key) { return !contains(keys, key); });
}
/***
 * Pick keys from `item` and return new object
 * @param item
 * @param keys The keys to include
 * @returns {Partial<T>}
 */
export function pick(item, keys) {
    return filter(item, function (key) { return contains(keys, key); });
}
var toString = Object.prototype.toString;
function isMergeableObject(val) {
    return val && typeof val === 'object'
        && toString.call(val) !== '[object RegExp]'
        && toString.call(val) !== '[object Date]';
}
function emptyTarget(val) {
    return isArray(val) ? [] : {};
}
function cloneIfNecessary(value) {
    return (isMergeableObject(value)) ? merge(emptyTarget(value), value) : value;
}
function arrayMerge(target, source) {
    var destination = target.slice();
    forEach(source, function (e, i) {
        if (typeof destination[i] === 'undefined') {
            destination[i] = cloneIfNecessary(e);
        }
        else if (isMergeableObject(e)) {
            destination[i] = merge(target[i], e);
        }
        else if (target.indexOf(e) === -1) {
            destination.push(cloneIfNecessary(e));
        }
    });
    return destination;
}
function mergeObject(target, source) {
    var destination = {};
    if (isMergeableObject(target)) {
        forEach(keys(target), function (key) {
            destination[key] = cloneIfNecessary(target[key]);
        });
    }
    forEach(keys(source), function (key) {
        if (!isMergeableObject(source[key]) || !target[key]) {
            destination[key] = cloneIfNecessary(source[key]);
        }
        else {
            destination[key] = merge(target[key], source[key]);
        }
    });
    return destination;
}
/***
 * This method is like Object.assign except that it recursively merges own and inherited enumerable string
 * keyed properties of source objects into the destination object.
 * Source properties that resolve to undefined are skipped if a destination value exists.
 * Array and plain object properties are merged recursively.
 * Other objects and value types are overridden by assignment.
 * Source objects are applied from left to right.
 * Subsequent sources overwrite property assignments of previous sources.
 * @param target
 * @param source
 * @returns {object|Array}
 */
export function merge(target, source) {
    if (isArray(source)) {
        return isArray(target) ? arrayMerge(target, source) : cloneIfNecessary(source);
    }
    else {
        return mergeObject(target, source);
    }
}
/***
 * Creates a patch for `old` using values from `patch`
 * @example `patch({a:1,b:2}, {a:1,b:3,c:4}) === {b:3,c:4}`
 * @param old
 * @param patch
 * @returns {Partial<Partial<any>>}
 */
export function patch(old, patch) {
    return filter(patch, function (key) { return patch[key] !== old[key]; });
}
/***
 * Check object keys equality
 * @param objA
 * @param objB
 * @returns {boolean}
 */
export function shallowEqual(objA, objB) {
    if (objA === objB) {
        return true;
    }
    if (typeof objA !== 'object' || !objA ||
        typeof objB !== 'object' || !objB) {
        return false;
    }
    var keysA = keys(objA);
    var keysB = keys(objB);
    if (keysA.length !== keysB.length) {
        return false;
    }
    var bHasOwnProperty = Object.prototype.hasOwnProperty.bind(objB);
    // Test for A's keys different from B.
    for (var idx = 0; idx < keysA.length; idx++) {
        var key = keysA[idx];
        if (!bHasOwnProperty(key)) {
            return false;
        }
        var valueA = objA[key];
        var valueB = objB[key];
        if (valueA !== valueB) {
            return false;
        }
    }
    return true;
}
//# sourceMappingURL=objects.js.map