import { curry } from './functions';
var asciiWords = /[^\x00-\x2f\x3a-\x40\x5b-\x60\x7b-\x7f]+/g;
/***
 * Spit string into words, unicode is not supported
 * @param str The string to split
 * @returns {RegExpMatchArray|[]} Matched words
 */
export function words(str) {
    return (str && str.match(asciiWords)) || [];
}
function mapWords(str, wordMapFn, join) {
    if (join === void 0) { join = ''; }
    return words(str).map(wordMapFn).join(join);
}
/***
 * Convert string to lowerCase
 * @param s The string to convert
 * @returns {string} Returns string in lowercase
 */
export function lowerCase(s) {
    return s.toLowerCase();
}
function caseStr(casing, str) {
    return mapWords(str, lowerCase, casing);
}
/***
 * Convert string to kebab-case
 * @param s The string to convert
 * @returns {string} Returns string in kebab-case
 */
export var kebabCase = curry(caseStr)('-');
/***
 * Convert string to snake_case
 * @param s The string to convert
 * @returns {string} Returns string in snake_case
 */
export var snakeCase = curry(caseStr)('_');
/***
 * Capitalize first char of string
 * @param str
 * @returns {string} Returns string with first char capitalized
 */
export function capitalize(str) {
    return str[0].toUpperCase() + str.slice(1);
}
/***
 * Convert string to camelCase
 * @param str The string to convert
 * @returns {string} Returns string in camelCase
 */
export function camelCase(str) {
    return mapWords(str, function (word, i) {
        word = lowerCase(word);
        return i === 0 ? word : capitalize(word);
    });
}
/***
 * Removes leading and trailing whitespace or specified characters from string.
 * @param str The string to trim.
 * @returns {string} Trimmed string
 */
export function trim(str) {
    return str.trim();
}
/***
 * Convert wildcard string to RegExp. Like `hel*o` becames `/hel.*o/`
 * @param wildcardPattern pattern to convert
 * @param flags flags for created regex 'igm'
 * @returns {RegExp} Constructed regexp
 */
export function wildcardToRegExp(wildcardPattern, flags) {
    var regexMatch = /^\/(.+)\/([im]+)?$/.exec(wildcardPattern);
    if (regexMatch) {
        return new RegExp(regexMatch[1], regexMatch[2] || flags);
    }
    var pattern = wildcardPattern
        .replace(/[\-\[\]\/{}()?.\\^$|]/g, '\\$&')
        .replace(/\*/g, '.*?')
        .replace(/\+/g, '.+?');
    return new RegExp("^" + pattern + "$", flags);
}
/***
 * Compile string template into template function.
 * Template language is similar to template strings in es6 `Hello ${name}`.
 * Doesn't support anything but property name inside `${}`
 * @example Use `template('Hello ${name}')(templateObject({name:'Bob'}))==='Hello Bob'`
 * @param template The template string
 * @returns {(data:<T>(key: keyof T) => string)=>string} Compiled template function.
 */
export function template(template) {
    var sanitized = template.split(/(\${([\s]*[^;\s{]+[\s]*)})/g);
    var fn = 'return [';
    for (var i = 0; i < sanitized.length; i++) {
        if (sanitized[i][0] === '$') {
            fn += "$d('" + sanitized[++i] + "'),";
        }
        else {
            fn += "\"" + sanitized[i].replace('"', '\\"') + "\",";
        }
    }
    return Function('$d', fn + "]");
}
/***
 * Function to use with `template` return object keys by template key
 * @param value
 * @param nullValue
 * @returns {(key:keyof T)=>string?}
 */
export function templateObject(value, nullValue) {
    return function (key) {
        return value[key] || nullValue;
    };
}
//# sourceMappingURL=strings.js.map